@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,7 +11,7 @@
 #include <asiolink/asiolink.h>
 #include <cc/data.h>
 #include <cc/command_interpreter.h>
-#include <database/database_connection.h>
+#include <util/reconnect_ctl.h>
 #include <dhcpsrv/timer_mgr.h>
 #include <dhcp4/dhcp4_srv.h>
 
@@ -33,7 +33,7 @@ public:
                         uint16_t client_port = 0);
 
     /// @brief Destructor.
-    ~ControlledDhcpv4Srv();
+    virtual ~ControlledDhcpv4Srv();
 
     /// @brief Initializes the server.
     ///
@@ -43,7 +43,7 @@ public:
     /// This method may throw if initialization fails.
     void init(const std::string& config_file);
 
-    /// @brief Loads specific config file
+    /// @brief Configure DHCPv4 server using the configuration file specified.
     ///
     /// This utility method is called whenever we know a filename of the config
     /// and need to load it. It calls config-set command once the content of
@@ -51,6 +51,12 @@ public:
     /// config-set handler will process the config file (load it as current
     /// configuration).
     ///
+    /// This function is used to both configure the DHCP server on its startup
+    /// and dynamically reconfigure the server when SIGHUP signal is received.
+    ///
+    /// It fetches DHCPv4 server's configuration from the 'Dhcp4' section of
+    /// the JSON configuration file.
+    ///
     /// @param file_name name of the file to be loaded
     /// @return status of the file loading and outcome of config-set
     isc::data::ConstElementPtr
@@ -63,7 +69,8 @@ public:
     void cleanup();
 
     /// @brief Initiates shutdown procedure for the whole DHCPv4 server.
-    void shutdown();
+    /// @param exit_value integer value to the process should exit with.
+    void shutdownServer(int exit_value);
 
     /// @brief Command processor
     ///
@@ -121,12 +128,12 @@ public:
         return (server_);
     }
 
-
 private:
+
     /// @brief Callback that will be called from iface_mgr when data
     /// is received over control socket.
     ///
-    /// This static callback method is called from IfaceMgr::receive6() method,
+    /// This static callback method is called from IfaceMgr::receive4() method,
     /// when there is a new command or configuration sent over control socket
     /// (that was sent from some yet unspecified sender).
     static void sessionReader(void);
@@ -205,8 +212,6 @@ private:
     /// @param command (parameter ignored)
     /// @param args configuration to be processed. Expected format:
     /// map containing Dhcp4 map that contains DHCPv4 server configuration.
-    /// May also contain Logging map that specifies logging configuration
-    /// for backward compatibility.
     ///
     /// @return status of the command
     isc::data::ConstElementPtr
@@ -220,8 +225,6 @@ private:
     /// @param command (parameter ignored)
     /// @param args configuration to be checked. Expected format:
     /// map containing Dhcp4 map that contains DHCPv4 server configuration.
-    /// May also contain Logging map that specifies logging configuration
-    /// for backward compatibility.
     ///
     /// @return status of the command
     isc::data::ConstElementPtr
@@ -231,7 +234,7 @@ private:
     /// @brief A handler for processing 'dhcp-disable' command.
     ///
     /// @param command command name (ignored).
-    /// @param args aguments for the command. It must be a map and
+    /// @param args arguments for the command. It must be a map and
     /// it may include optional 'max-period' parameter.
     ///
     /// @return result of the command.
@@ -249,7 +252,6 @@ private:
     commandDhcpEnableHandler(const std::string& command,
                              isc::data::ConstElementPtr args);
 
-
     /// @Brief handler for processing 'version-get' command
     ///
     /// This handler processes version-get command, which returns
@@ -304,6 +306,19 @@ private:
     commandServerTagGetHandler(const std::string& command,
                                isc::data::ConstElementPtr args);
 
+    /// @brief handler for config-backend-pull command
+    ///
+    /// This method handles the config-backend-pull command, which updates
+    /// the server configuration from the Config Backends immediately.
+    ///
+    /// @param command (parameter ignored)
+    /// @param args (ignored)
+    ///
+    /// @return status of the command/
+    isc::data::ConstElementPtr
+    commandConfigBackendPullHandler(const std::string& command,
+                                    isc::data::ConstElementPtr args);
+
     /// @brief handler for processing 'status-get' command
     ///
     /// This handler processes status-get command, which retrieves
@@ -316,6 +331,32 @@ private:
     commandStatusGetHandler(const std::string& command,
                             isc::data::ConstElementPtr args);
 
+    /// @brief handler for processing 'statistic-sample-count-set-all' command
+    ///
+    /// This handler processes statistic-sample-count-set-all command,
+    /// which sets max_sample_count_ limit of all statistics and the default.
+    /// @ref isc::stats::StatsMgr::statisticSetMaxSampleCountAllHandler
+    ///
+    /// @param command (ignored)
+    /// @param args structure containing a map that contains "max-samples"
+    /// @return process information wrapped in a response
+    isc::data::ConstElementPtr
+    commandStatisticSetMaxSampleCountAllHandler(const std::string& command,
+                                                isc::data::ConstElementPtr args);
+
+    /// @brief handler for processing 'statistic-sample-age-set-all' command
+    ///
+    /// This handler processes statistic-sample-age-set-all command,
+    /// which sets max_sample_age_ limit of all statistics and the default.
+    /// @ref isc::stats::StatsMgr::statisticSetMaxSampleAgeAllHandler
+    ///
+    /// @param command (ignored)
+    /// @param args structure containing a map that contains "duration"
+    /// @return process information wrapped in a response
+    isc::data::ConstElementPtr
+    commandStatisticSetMaxSampleAgeAllHandler(const std::string& command,
+                                              isc::data::ConstElementPtr args);
+
     /// @brief Reclaims expired IPv4 leases and reschedules timer.
     ///
     /// This is a wrapper method for @c AllocEngine::reclaimExpiredLeases4.
@@ -346,27 +387,8 @@ private:
     /// deleted.
     void deleteExpiredReclaimedLeases(const uint32_t secs);
 
-    /// @brief Attempts to reconnect the server to the DB backend managers
-    ///
-    /// This is a self-rescheduling function that attempts to reconnect to the
-    /// server's DB backends after connectivity to one or more have been
-    /// lost.  Upon entry it will attempt to reconnect via @ref CfgDdbAccess::
-    /// createManagers.  If this is succesful, DHCP servicing is re-enabled and
-    /// server returns to normal operation.
-    ///
-    /// If reconnection fails and the maximum number of retries has not been
-    /// exhausted, it will schedule a call to itself to occur at the
-    /// configured retry interval. DHCP service remains disabled.
-    ///
-    /// If the maximum number of retries has been exhausted an error is logged
-    /// and the server shuts down.
-    ///
-    /// @param db_reconnect_ctl pointer to the ReconnectCtl containing the
-    /// configured reconnect parameters
-    ///
-    void dbReconnect(db::ReconnectCtlPtr db_reconnect_ctl);
-
-    /// @brief Callback DB backends should invoke upon loss of connectivity
+    /// @brief Callback DB backends should be invoked upon loss of the
+    /// connectivity.
     ///
     /// This function is invoked by DB backends when they detect a loss of
     /// connectivity.  The parameter, db_reconnect_ctl, conveys the configured
@@ -375,16 +397,48 @@ private:
     ///
     /// If either value is zero, reconnect is presumed to be disabled and
     /// the function will schedule a shutdown and return false.  This instructs
-    /// the DB backend layer (the caller) to treat the connectivity loss as fatal.
-    ///
-    /// Otherwise, the function saves db_reconnect_ctl and invokes
-    /// dbReconnect to initiate the reconnect process.
+    /// the DB backend layer (the caller) to treat the connectivity loss as
+    /// fatal. It stops the DHCP service until the connection is recovered.
     ///
     /// @param db_reconnect_ctl pointer to the ReconnectCtl containing the
     /// configured reconnect parameters
     ///
     /// @return false if reconnect is not configured, true otherwise
-    bool dbLostCallback(db::ReconnectCtlPtr db_reconnect_ctl);
+    bool dbLostCallback(util::ReconnectCtlPtr db_reconnect_ctl);
+
+    /// @brief Callback DB backends should be invoked upon restoration of
+    /// connectivity.
+    ///
+    /// This function is invoked by DB backends when they recover the
+    /// connectivity. It starts the DHCP service after the connection is
+    /// recovered.
+    ///
+    /// @param db_reconnect_ctl pointer to the ReconnectCtl containing the
+    /// configured reconnect parameters
+    ///
+    /// @return false if reconnect is not configured, true otherwise
+    bool dbRecoveredCallback(util::ReconnectCtlPtr db_reconnect_ctl);
+
+    /// @brief Callback DB backends should be invoked upon failing to restore
+    /// connectivity.
+    ///
+    /// This function is invoked by DB backends when they fail to recover the
+    /// connectivity. It stops the server.
+    ///
+    /// @param db_reconnect_ctl pointer to the ReconnectCtl containing the
+    /// configured reconnect parameters
+    ///
+    /// @return false if reconnect is not configured, true otherwise
+    bool dbFailedCallback(util::ReconnectCtlPtr db_reconnect_ctl);
+
+    /// @brief This callback should be invoked upon failing to bind sockets.
+    ///
+    /// This function is invoked during the configuration of the interfaces
+    /// when they fail to bind the service sockets. It may stop the server.
+    ///
+    /// @param reconnect_ctl pointer to the ReconnectCtl containing the
+    /// configured reconnect parameters
+    void openSocketsFailedCallback(util::ReconnectCtlPtr reconnect_ctl);
 
     /// @brief Callback invoked periodically to fetch configuration updates
     /// from the Config Backends.
@@ -403,12 +457,9 @@ private:
     /// @brief Static pointer to the sole instance of the DHCP server.
     ///
     /// This is required for config and command handlers to gain access to
-    /// the server
+    /// the server. Some of them need to be static methods.
     static ControlledDhcpv4Srv* server_;
 
-    /// @brief IOService object, used for all ASIO operations.
-    isc::asiolink::IOService io_service_;
-
     /// @brief Instance of the @c TimerMgr.
     ///
     /// Shared pointer to the instance of timer @c TimerMgr is held here to
@@ -416,7 +467,7 @@ private:
     TimerMgrPtr timer_mgr_;
 };
 
-}; // namespace isc::dhcp
-}; // namespace isc
+}  // namespace dhcp
+}  // namespace isc
 
 #endif
