@@ -1,4 +1,4 @@
-// Copyright (C) 2021-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2021-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -81,7 +81,10 @@ CmdHttpListener::start() {
             .arg(port_)
             .arg(tls_context_ ? "true" : "false");
     } catch (const std::exception& ex) {
-        isc_throw(Unexpected, "CmdHttpListener::run failed:" << ex.what());
+        thread_io_service_.reset();
+        http_listener_.reset();
+        thread_pool_.reset();
+        isc_throw(Unexpected, "CmdHttpListener::start failed: " << ex.what());
     }
 }
 
