@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -74,7 +74,7 @@ public:
 
 private:
 
-    /// @brief Holds a pointers to the packets.
+    /// @brief Holds a pair of pointers of the packets.
     std::pair<PktTypePtr, PktTypePtr> pkts_;
 };
 
@@ -328,6 +328,11 @@ protected:
 
 public:
 
+    /// @brief Clones all options so that they can be safely modified.
+    ///
+    /// @return A container with option clones.
+    OptionCollection cloneOptions();
+
     /// @brief Returns the first option of specified type.
     ///
     /// Returns the first option of specified type. Note that in DHCPv6 several
@@ -480,7 +485,7 @@ public:
     /// @brief Sets interface index.
     ///
     /// @param ifindex specifies interface index.
-    void setIndex(uint32_t ifindex) {
+    void setIndex(int ifindex) {
         ifindex_ = ifindex;
     };
 
@@ -492,7 +497,7 @@ public:
     /// @brief Returns interface index.
     ///
     /// @return interface index
-    uint32_t getIndex() const {
+    int getIndex() const {
         return (ifindex_);
     };
 
@@ -516,8 +521,8 @@ public:
     /// Sets interface name over which packet was received or is
     /// going to be transmitted.
     ///
-    /// @return interface name
-    void setIface(const std::string& iface ) { iface_ = iface; };
+    /// @param iface The interface name
+    void setIface(const std::string& iface) { iface_ = iface; };
 
     /// @brief Sets remote hardware address.
     ///
@@ -796,7 +801,92 @@ private:
 /// @brief A pointer to either Pkt4 or Pkt6 packet
 typedef boost::shared_ptr<isc::dhcp::Pkt> PktPtr;
 
-}; // namespace isc::dhcp
-}; // namespace isc
+/// @brief RAII object enabling duplication of the stored options and restoring
+/// the original options on destructor.
+///
+/// This object enables duplication of the stored options and restoring the
+/// original options on destructor. When the object goes out of scope, the
+/// initial options are restored. This is applicable in cases when the server is
+/// going to invoke a callout (hook library) where the list of options in the
+/// packet will be modified. This can also be used to restore the initial
+/// suboptions of an option when the suboptions are changed (e.g. when splitting
+/// long options and suboptions). The use of RAII object eliminates the need for
+/// explicitly copying and restoring the list of options and is safer in case of
+/// exceptions thrown by callouts and a presence of multiple exit points.
+class ScopedSubOptionsCopy {
+public:
+
+    /// @brief Constructor.
+    ///
+    /// Creates a copy of the initial options on an option.
+    ///
+    /// @param opt Pointer to the option.
+    ScopedSubOptionsCopy(const OptionPtr& opt) : option_(opt) {
+        if (opt) {
+            options_ = opt->getMutableOptions();
+        }
+    }
+
+    /// @brief Destructor.
+    ///
+    /// Restores the initial options on a packet.
+    ~ScopedSubOptionsCopy() {
+        if (option_) {
+            option_->getMutableOptions() = options_;
+        }
+    }
+
+private:
+
+    /// @brief Holds a pointer to the option.
+    OptionPtr option_;
+
+    /// @brief Holds the initial options.
+    OptionCollection options_;
+};
+
+/// @brief RAII object enabling duplication of the stored options and restoring
+/// the original options on destructor.
+///
+/// This object enables duplication of the stored options and restoring the
+/// original options on destructor. When the object goes out of scope, the
+/// initial options are restored. This is applicable in cases when the server is
+/// going to invoke a callout (hook library) where the list of options in the
+/// packet will be modified. The use of RAII object eliminates the need for
+/// explicitly copying and restoring the list of options and is safer in case of
+/// exceptions thrown by callouts and a presence of multiple exit points.
+///
+/// @tparam PktType Type of the packet, e.g. Pkt4, Pkt6, Pkt4o6.
+template<typename PktType>
+class ScopedPktOptionsCopy {
+public:
+
+    /// @brief Constructor.
+    ///
+    /// Creates a copy of the initial options on a packet.
+    ///
+    /// @param pkt Pointer to the packet.
+    ScopedPktOptionsCopy(PktType& pkt) : pkt_(pkt), options_(pkt.options_) {
+        pkt_.options_ = pkt_.cloneOptions();
+    }
+
+    /// @brief Destructor.
+    ///
+    /// Restores the initial options on a packet.
+    ~ScopedPktOptionsCopy() {
+        pkt_.options_ = options_;
+    }
+
+private:
+
+    /// @brief Holds a reference to the packet.
+    PktType& pkt_;
+
+    /// @brief Holds the initial options.
+    OptionCollection options_;
+};
+
+} // end of namespace isc::dhcp
+} // end of namespace isc
 
 #endif
