@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,6 +8,7 @@
 #include <asiolink/io_address.h>
 #include <dhcp/dhcp6.h>
 #include <dhcp/hwaddr.h>
+#include <dhcp/libdhcp++.h>
 #include <dhcp/protocol_util.h>
 #include <util/buffer.h>
 #include <gtest/gtest.h>
@@ -381,4 +382,182 @@ TEST(ProtocolUtilTest, writeIpUdpHeader) {
     EXPECT_EQ(0x8817, udp_checksum);
 }
 
+/// Test that checks the RAII implementation of ScopedEnableOptionsCopy works
+/// as expected, restoring the copy retrieve options flag.
+TEST(ScopedEnableOptionsCopy, enableOptionsCopy) {
+    Pkt4Ptr pkt(new Pkt4(DHCPDISCOVER, 2543));
+    OptionPtr option = Option::create(Option::V4, DHO_BOOT_FILE_NAME);
+    pkt->addOption(option);
+    ASSERT_FALSE(pkt->isCopyRetrievedOptions());
+    ASSERT_EQ(option, pkt->getOption(DHO_BOOT_FILE_NAME));
+    {
+        ScopedEnableOptionsCopy<Pkt4> oc(pkt);
+        ASSERT_TRUE(pkt->isCopyRetrievedOptions());
+        OptionPtr option_copy = pkt->getOption(DHO_BOOT_FILE_NAME);
+        ASSERT_NE(option, option_copy);
+        option = option_copy;
+    }
+    ASSERT_FALSE(pkt->isCopyRetrievedOptions());
+    ASSERT_EQ(option, pkt->getOption(DHO_BOOT_FILE_NAME));
+    {
+        try {
+            ScopedEnableOptionsCopy<Pkt4> oc(pkt);
+            ASSERT_TRUE(pkt->isCopyRetrievedOptions());
+            OptionPtr option_copy = pkt->getOption(DHO_BOOT_FILE_NAME);
+            ASSERT_NE(option, option_copy);
+            option = option_copy;
+            throw 0;
+        } catch (...) {
+            ASSERT_FALSE(pkt->isCopyRetrievedOptions());
+            ASSERT_EQ(option, pkt->getOption(DHO_BOOT_FILE_NAME));
+        }
+        ASSERT_FALSE(pkt->isCopyRetrievedOptions());
+        ASSERT_EQ(option, pkt->getOption(DHO_BOOT_FILE_NAME));
+    }
+}
+
+/// Test that checks the RAII implementation of ScopedPkt4OptionsCopy works
+/// as expected, restoring the initial Pkt4 options.
+TEST(ScopedOptionsCopy, pkt4OptionsCopy) {
+    Pkt4Ptr pkt(new Pkt4(DHCPDISCOVER, 2543));
+    OptionPtr option = Option::create(Option::V4, DHO_BOOT_FILE_NAME);
+    pkt->addOption(option);
+    OptionCollection options = pkt->options_;
+    size_t count = options.size();
+    ASSERT_NE(0, count);
+    ASSERT_EQ(option, pkt->getOption(DHO_BOOT_FILE_NAME));
+    std::string expected = pkt->toText();
+    pkt->pack();
+    OutputBuffer buf = pkt->getBuffer();
+    {
+        ScopedPkt4OptionsCopy oc(*pkt);
+        ASSERT_NE(pkt->options_, options);
+        ASSERT_NE(option, pkt->getOption(DHO_BOOT_FILE_NAME));
+        pkt->pack();
+        ASSERT_EQ(buf.getLength(), pkt->getBuffer().getLength());
+        for (size_t index = 0; index < buf.getLength(); ++index) {
+            ASSERT_EQ(buf[index], pkt->getBuffer()[index]);
+        }
+        ASSERT_EQ(expected, pkt->toText());
+        pkt->delOption(DHO_BOOT_FILE_NAME);
+        ASSERT_EQ(pkt->options_.size(), count - 1);
+        ASSERT_FALSE(pkt->getOption(DHO_BOOT_FILE_NAME));
+    }
+    ASSERT_EQ(pkt->options_, options);
+    ASSERT_EQ(pkt->getOption(DHO_BOOT_FILE_NAME), option);
+    {
+        try {
+            ScopedPkt4OptionsCopy oc(*pkt);
+            ASSERT_NE(pkt->options_, options);
+            ASSERT_NE(option, pkt->getOption(DHO_BOOT_FILE_NAME));
+            pkt->pack();
+            ASSERT_EQ(buf.getLength(), pkt->getBuffer().getLength());
+            for (size_t index = 0; index < buf.getLength(); ++index) {
+                ASSERT_EQ(buf[index], pkt->getBuffer()[index]);
+            }
+            ASSERT_EQ(expected, pkt->toText());
+            pkt->delOption(DHO_BOOT_FILE_NAME);
+            ASSERT_EQ(pkt->options_.size(), count - 1);
+            ASSERT_FALSE(pkt->getOption(DHO_BOOT_FILE_NAME));
+            throw 0;
+        } catch (...) {
+            ASSERT_EQ(pkt->options_, options);
+            ASSERT_EQ(pkt->getOption(DHO_BOOT_FILE_NAME), option);
+        }
+        ASSERT_EQ(pkt->options_, options);
+        ASSERT_EQ(pkt->getOption(DHO_BOOT_FILE_NAME), option);
+    }
+}
+
+/// Test that checks the RAII implementation of ScopedPkt6OptionsCopy works
+/// as expected, restoring the initial Pkt6 options.
+TEST(ScopedOptionsCopy, pkt6OptionsCopy) {
+    Pkt6Ptr pkt(new Pkt6(DHCPV6_SOLICIT, 2543));
+    OptionPtr option = Option::create(Option::V6, D6O_BOOTFILE_URL);
+    pkt->addOption(option);
+    OptionCollection options = pkt->options_;
+    size_t count = options.size();
+    ASSERT_NE(0, count);
+    ASSERT_EQ(option, pkt->getOption(D6O_BOOTFILE_URL));
+    std::string expected = pkt->toText();
+    pkt->pack();
+    OutputBuffer buf = pkt->getBuffer();
+    {
+        ScopedPkt6OptionsCopy oc(*pkt);
+        ASSERT_NE(pkt->options_, options);
+        ASSERT_NE(option, pkt->getOption(D6O_BOOTFILE_URL));
+        pkt->pack();
+        ASSERT_EQ(buf.getLength(), pkt->getBuffer().getLength());
+        for (size_t index = 0; index < buf.getLength(); ++index) {
+            ASSERT_EQ(buf[index], pkt->getBuffer()[index]);
+        }
+        ASSERT_EQ(expected, pkt->toText());
+        pkt->delOption(D6O_BOOTFILE_URL);
+        ASSERT_EQ(pkt->options_.size(), count - 1);
+        ASSERT_FALSE(pkt->getOption(D6O_BOOTFILE_URL));
+    }
+    ASSERT_EQ(pkt->options_, options);
+    ASSERT_EQ(pkt->getOption(D6O_BOOTFILE_URL), option);
+    {
+        try {
+            ScopedPkt6OptionsCopy oc(*pkt);
+            ASSERT_NE(pkt->options_, options);
+            ASSERT_NE(option, pkt->getOption(D6O_BOOTFILE_URL));
+            pkt->pack();
+            ASSERT_EQ(buf.getLength(), pkt->getBuffer().getLength());
+            for (size_t index = 0; index < buf.getLength(); ++index) {
+                ASSERT_EQ(buf[index], pkt->getBuffer()[index]);
+            }
+            ASSERT_EQ(expected, pkt->toText());
+            pkt->delOption(D6O_BOOTFILE_URL);
+            ASSERT_EQ(pkt->options_.size(), count - 1);
+            ASSERT_FALSE(pkt->getOption(D6O_BOOTFILE_URL));
+            throw 0;
+        } catch (...) {
+            ASSERT_EQ(pkt->options_, options);
+            ASSERT_EQ(pkt->getOption(D6O_BOOTFILE_URL), option);
+        }
+        ASSERT_EQ(pkt->options_, options);
+        ASSERT_EQ(pkt->getOption(D6O_BOOTFILE_URL), option);
+    }
+}
+
+/// Test that checks the RAII implementation of ScopedSubOptionsCopy works
+/// as expected, restoring the initial option suboptions.
+TEST(ScopedOptionsCopy, subOptionsCopy) {
+    OptionPtr initial = Option::create(Option::V4, 231);
+    OptionPtr option = Option::create(Option::V4, DHO_BOOT_FILE_NAME);
+    initial->addOption(option);
+    OptionCollection options = initial->getOptions();
+    size_t count = options.size();
+    ASSERT_NE(0, count);
+    ASSERT_EQ(option, initial->getOption(DHO_BOOT_FILE_NAME));
+    {
+        ScopedSubOptionsCopy oc(initial);
+        ASSERT_EQ(initial->getOptions(), options);
+        ASSERT_EQ(option, initial->getOption(DHO_BOOT_FILE_NAME));
+        initial->delOption(DHO_BOOT_FILE_NAME);
+        ASSERT_EQ(initial->getOptions().size(), count - 1);
+        ASSERT_FALSE(initial->getOption(DHO_BOOT_FILE_NAME));
+    }
+    ASSERT_EQ(initial->getOptions(), options);
+    ASSERT_EQ(initial->getOption(DHO_BOOT_FILE_NAME), option);
+    {
+        try {
+            ScopedSubOptionsCopy oc(initial);
+            ASSERT_EQ(initial->getOptions(), options);
+            ASSERT_EQ(option, initial->getOption(DHO_BOOT_FILE_NAME));
+            initial->delOption(DHO_BOOT_FILE_NAME);
+            ASSERT_EQ(initial->getOptions().size(), count - 1);
+            ASSERT_FALSE(initial->getOption(DHO_BOOT_FILE_NAME));
+            throw 0;
+        } catch (...) {
+            ASSERT_EQ(initial->getOptions(), options);
+            ASSERT_EQ(initial->getOption(DHO_BOOT_FILE_NAME), option);
+        }
+        ASSERT_EQ(initial->getOptions(), options);
+        ASSERT_EQ(initial->getOption(DHO_BOOT_FILE_NAME), option);
+    }
+}
+
 } // anonymous namespace
