@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -38,7 +38,7 @@ CfgSubnets6::add(const Subnet6Ptr& subnet) {
 
     LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE, DHCPSRV_CFGMGR_ADD_SUBNET6)
               .arg(subnet->toText());
-    static_cast<void>(subnets_.push_back(subnet));
+    static_cast<void>(subnets_.insert(subnet));
 }
 
 Subnet6Ptr
@@ -93,13 +93,11 @@ CfgSubnets6::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks6Ptr networks,
     // Iterate over the subnets to be merged. They will replace the existing
     // subnets with the same id. All new subnets will be inserted into the
     // configuration into which we're merging.
-    auto other_subnets = other.getAll();
-    for (auto other_subnet = other_subnets->begin();
-         other_subnet != other_subnets->end();
-         ++other_subnet) {
+    auto const& other_subnets = other.getAll();
+    for (auto const& other_subnet : *other_subnets) {
 
         // Check if there is a subnet with the same ID.
-        auto subnet_it = index_id.find((*other_subnet)->getID());
+        auto subnet_it = index_id.find(other_subnet->getID());
         if (subnet_it != index_id.end()) {
 
             // Subnet found.
@@ -107,7 +105,7 @@ CfgSubnets6::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks6Ptr networks,
 
             // If the existing subnet and other subnet
             // are the same instance skip it.
-            if (existing_subnet == *other_subnet) {
+            if (existing_subnet == other_subnet) {
                 continue;
             }
 
@@ -125,7 +123,7 @@ CfgSubnets6::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks6Ptr networks,
         }
 
         // Check if there is a subnet with the same prefix.
-        auto subnet_prefix_it = index_prefix.find((*other_subnet)->toText());
+        auto subnet_prefix_it = index_prefix.find(other_subnet->toText());
         if (subnet_prefix_it != index_prefix.end()) {
 
             // Subnet found.
@@ -149,32 +147,33 @@ CfgSubnets6::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks6Ptr networks,
         }
 
         // Create the subnet's options based on the given definitions.
-        (*other_subnet)->getCfgOption()->createOptions(cfg_def);
+        other_subnet->getCfgOption()->createOptions(cfg_def);
 
         // Create the options for pool based on the given definitions.
-        for (auto pool : (*other_subnet)->getPoolsWritable(Lease::TYPE_NA)) {
+        for (auto const& pool : other_subnet->getPoolsWritable(Lease::TYPE_NA)) {
             pool->getCfgOption()->createOptions(cfg_def);
         }
 
-        for (auto pool : (*other_subnet)->getPoolsWritable(Lease::TYPE_PD)) {
+        // Create the options for pd pool based on the given definitions.
+        for (auto const& pool : other_subnet->getPoolsWritable(Lease::TYPE_PD)) {
             pool->getCfgOption()->createOptions(cfg_def);
         }
 
         // Add the "other" subnet to the our collection of subnets.
-        static_cast<void>(subnets_.push_back(*other_subnet));
+        static_cast<void>(subnets_.insert(other_subnet));
 
         // If it belongs to a shared network, find the network and
         // add the subnet to it
-        std::string network_name = (*other_subnet)->getSharedNetworkName();
+        std::string network_name = other_subnet->getSharedNetworkName();
         if (!network_name.empty()) {
             SharedNetwork6Ptr network = networks->getByName(network_name);
             if (network) {
-                network->add(*other_subnet);
+                network->add(other_subnet);
             } else {
                 // This implies the shared-network collection we were given
                 // is out of sync with the subnets we were given.
                 isc_throw(InvalidOperation, "Cannot assign subnet ID of "
-                          << (*other_subnet)->getID()
+                          << other_subnet->getID()
                           << " to shared network: " << network_name
                           << ", network does not exist");
             }
@@ -244,7 +243,6 @@ CfgSubnets6::selectSubnet(const SubnetSelector& selector) const {
 
     // If relay agent link address is set, we're dealing with a relayed message.
     } else {
-
         // Find the subnet using the Interface Id option, if present.
         subnet = selectSubnet(selector.interface_id_, selector.client_classes_);
 
@@ -265,79 +263,79 @@ Subnet6Ptr
 CfgSubnets6::selectSubnet(const asiolink::IOAddress& address,
                           const ClientClasses& client_classes,
                           const bool is_relay_address) const {
-
     // If the specified address is a relay address we first need to match
     // it with the relay addresses specified for all subnets.
     if (is_relay_address) {
-        for (Subnet6Collection::const_iterator subnet = subnets_.begin();
-             subnet != subnets_.end(); ++subnet) {
+        for (auto const& subnet : subnets_) {
 
             // If the specified address matches a relay address, return this
             // subnet.
-            if ((*subnet)->hasRelays()) {
-                if (!(*subnet)->hasRelayAddress(address)) {
+            if (subnet->hasRelays()) {
+                if (!subnet->hasRelayAddress(address)) {
                     continue;
                 }
 
             } else {
                 SharedNetwork6Ptr network;
-                (*subnet)->getSharedNetwork(network);
+                subnet->getSharedNetwork(network);
                 if (!network || !network->hasRelayAddress(address)) {
                     continue;
                 }
             }
 
-            if ((*subnet)->clientSupported(client_classes)) {
+            if (subnet->clientSupported(client_classes)) {
                 // The relay address is matching the one specified for a subnet
                 // or its shared network.
                 LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
                           DHCPSRV_CFGMGR_SUBNET6_RELAY)
-                    .arg((*subnet)->toText()).arg(address.toText());
-                return (*subnet);
+                    .arg(subnet->toText()).arg(address.toText());
+                return (subnet);
             }
         }
     }
 
     // No success so far. Check if the specified address is in range
     // with any subnet.
-    for (Subnet6Collection::const_iterator subnet = subnets_.begin();
-         subnet != subnets_.end(); ++subnet) {
-        if ((*subnet)->inRange(address) &&
-            (*subnet)->clientSupported(client_classes)) {
+    for (auto const& subnet : subnets_) {
+        if (subnet->inRange(address) && subnet->clientSupported(client_classes)) {
             LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE, DHCPSRV_CFGMGR_SUBNET6)
-                      .arg((*subnet)->toText()).arg(address.toText());
-            return (*subnet);
+                      .arg(subnet->toText()).arg(address.toText());
+            return (subnet);
         }
     }
 
+    LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+              DHCPSRV_SUBNET6_SELECT_BY_ADDRESS_NO_MATCH)
+        .arg(address.toText());
+
     // Nothing found.
     return (Subnet6Ptr());
 }
 
-
 Subnet6Ptr
 CfgSubnets6::selectSubnet(const std::string& iface_name,
                           const ClientClasses& client_classes) const {
-
     // If empty interface specified, we can't select subnet by interface.
     if (!iface_name.empty()) {
-        for (Subnet6Collection::const_iterator subnet = subnets_.begin();
-             subnet != subnets_.end(); ++subnet) {
+        for (auto const& subnet : subnets_) {
 
             // If interface name matches with the one specified for the subnet
             // and the client is not rejected based on the classification,
             // return the subnet.
-            if (((*subnet)->getIface() == iface_name) &&
-                (*subnet)->clientSupported(client_classes)) {
-
+            if ((subnet->getIface() == iface_name) &&
+                subnet->clientSupported(client_classes)) {
                 LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
                           DHCPSRV_CFGMGR_SUBNET6_IFACE)
-                    .arg((*subnet)->toText()).arg(iface_name);
-                return (*subnet);
+                    .arg(subnet->toText()).arg(iface_name);
+                return (subnet);
             }
         }
     }
 
+    LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+              DHCPSRV_SUBNET6_SELECT_BY_INTERFACE_NO_MATCH)
+        .arg(iface_name);
+
     // No subnet found for this interface name.
     return (Subnet6Ptr());
 }
@@ -348,34 +346,37 @@ CfgSubnets6::selectSubnet(const OptionPtr& interface_id,
     // We can only select subnet using an interface id, if the interface
     // id is known.
     if (interface_id) {
-        for (Subnet6Collection::const_iterator subnet = subnets_.begin();
-             subnet != subnets_.end(); ++subnet) {
+        for (auto const& subnet : subnets_) {
 
             // If interface id matches for the subnet and the subnet is not
             // rejected based on the classification.
-            if ((*subnet)->getInterfaceId() &&
-                (*subnet)->getInterfaceId()->equals(interface_id) &&
-                (*subnet)->clientSupported(client_classes)) {
+            if (subnet->getInterfaceId() &&
+                subnet->getInterfaceId()->equals(interface_id) &&
+                subnet->clientSupported(client_classes)) {
 
                 LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
                       DHCPSRV_CFGMGR_SUBNET6_IFACE_ID)
-                    .arg((*subnet)->toText());
-                return (*subnet);
+                    .arg(subnet->toText());
+                return (subnet);
             }
         }
+
+        LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+                  DHCPSRV_SUBNET6_SELECT_BY_INTERFACE_ID_NO_MATCH)
+            .arg(interface_id->toText());
     }
+
     // No subnet found.
     return (Subnet6Ptr());
 }
 
 Subnet6Ptr
 CfgSubnets6::getSubnet(const SubnetID id) const {
-
     /// @todo: Once this code is migrated to multi-index container, use
     /// an index rather than full scan.
-    for (auto subnet = subnets_.begin(); subnet != subnets_.end(); ++subnet) {
-        if ((*subnet)->getID() == id) {
-            return (*subnet);
+    for (auto const& subnet : subnets_) {
+        if (subnet->getID() == id) {
+            return (subnet);
         }
     }
     return (Subnet6Ptr());
@@ -387,24 +388,29 @@ CfgSubnets6::removeStatistics() {
 
     StatsMgr& stats_mgr = StatsMgr::instance();
     // For each v6 subnet currently configured, remove the statistics.
-    for (Subnet6Collection::const_iterator subnet6 = subnets_.begin();
-         subnet6 != subnets_.end(); ++subnet6) {
-        SubnetID subnet_id = (*subnet6)->getID();
+    for (auto const& subnet6 : subnets_) {
+        SubnetID subnet_id = subnet6->getID();
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id, "total-nas"));
 
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "assigned-nas"));
 
+        stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                             "cumulative-assigned-nas"));
+
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id, "total-pds"));
 
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "assigned-pds"));
 
+        stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                             "cumulative-assigned-pds"));
+
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "declined-addresses"));
 
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
-                                             "declined-reclaimed-addresses"));
+                                             "reclaimed-declined-addresses"));
 
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "reclaimed-leases"));
@@ -417,24 +423,35 @@ CfgSubnets6::updateStatistics() {
 
     StatsMgr& stats_mgr = StatsMgr::instance();
     // For each v6 subnet currently configured, calculate totals
-    for (Subnet6Collection::const_iterator subnet6 = subnets_.begin();
-         subnet6 != subnets_.end(); ++subnet6) {
-        SubnetID subnet_id = (*subnet6)->getID();
+    for (auto const& subnet6 : subnets_) {
+        SubnetID subnet_id = subnet6->getID();
 
         stats_mgr.setValue(StatsMgr::generateName("subnet", subnet_id,
                                                   "total-nas"),
                            static_cast<int64_t>
-                           ((*subnet6)->getPoolCapacity(Lease::TYPE_NA)));
+                           (subnet6->getPoolCapacity(Lease::TYPE_NA)));
 
         stats_mgr.setValue(StatsMgr::generateName("subnet", subnet_id,
                                                   "total-pds"),
                             static_cast<int64_t>
-                            ((*subnet6)->getPoolCapacity(Lease::TYPE_PD)));
+                            (subnet6->getPoolCapacity(Lease::TYPE_PD)));
+
+        const std::string& name_nas =
+            StatsMgr::generateName("subnet", subnet_id, "cumulative-assigned-nas");
+        if (!stats_mgr.getObservation(name_nas)) {
+            stats_mgr.setValue(name_nas, static_cast<int64_t>(0));
+        }
+
+        const std::string& name_pds =
+            StatsMgr::generateName("subnet", subnet_id, "cumulative-assigned-pds");
+        if (!stats_mgr.getObservation(name_pds)) {
+            stats_mgr.setValue(name_pds, static_cast<int64_t>(0));
+        }
     }
 
     // Only recount the stats if we have subnets.
     if (subnets_.begin() != subnets_.end()) {
-            LeaseMgrFactory::instance().recountLeaseStats6();
+        LeaseMgrFactory::instance().recountLeaseStats6();
     }
 }
 
@@ -442,9 +459,8 @@ ElementPtr
 CfgSubnets6::toElement() const {
     ElementPtr result = Element::createList();
     // Iterate subnets
-    for (Subnet6Collection::const_iterator subnet = subnets_.cbegin();
-         subnet != subnets_.cend(); ++subnet) {
-        result->add((*subnet)->toElement());
+    for (auto const& subnet : subnets_) {
+        result->add(subnet->toElement());
     }
     return (result);
 }
