@@ -1,9 +1,11 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <hooks/callout_handle.h>
 #include <hooks/callout_manager.h>
 #include <hooks/library_manager.h>
@@ -16,6 +18,8 @@
 #include <log/message_dictionary.h>
 #include <log/message_initializer.h>
 
+#include <util/multi_threading_mgr.h>
+
 #include <gtest/gtest.h>
 
 #include <algorithm>
@@ -28,6 +32,7 @@
 using namespace isc;
 using namespace isc::hooks;
 using namespace isc::log;
+using namespace isc::util;
 using namespace std;
 
 namespace {
@@ -47,6 +52,9 @@ public:
 
         // Ensure the marker file is not present at the start of a test.
         static_cast<void>(remove(MARKER_FILE));
+
+        // Disable multi-threading.
+        MultiThreadingMgr::instance().setMode(false);
     }
 
     /// @brief Destructor
@@ -54,6 +62,9 @@ public:
     /// Ensures a marker file is removed after each test.
     ~LibraryManagerTest() {
         static_cast<void>(remove(MARKER_FILE));
+
+        // Disable multi-threading.
+        MultiThreadingMgr::instance().setMode(false);
     }
 
     /// @brief Marker file present
@@ -93,6 +104,17 @@ public:
                                                   r1, d2, r2, d3, r3);
     }
 
+    /// @brief Call command handlers test.
+    ///
+    /// A wrapper around the method of the same name in the HooksCommonTestClass
+    /// object, this passes this class's CalloutManager to that method.
+    ///
+    /// @param r1..r2, d1..d2 Values and intermediate values expected.
+    void executeCallCommandHandlers(int d1, int r1, int d2, int r2) {
+        HooksCommonTestClass::executeCallCommandHandlers(callout_manager_,
+                                                         d1, r1, d2, r2);
+    }
+
     /// Callout manager used for the test.
     boost::shared_ptr<CalloutManager> callout_manager_;
 };
@@ -119,15 +141,25 @@ public:
     {}
 
     /// Public methods that call protected methods on the superclass.
+    using LibraryManager::unloadLibrary;
     using LibraryManager::openLibrary;
     using LibraryManager::closeLibrary;
     using LibraryManager::checkVersion;
+    using LibraryManager::checkMultiThreadingCompatible;
     using LibraryManager::registerStandardCallouts;
     using LibraryManager::runLoad;
-    using LibraryManager::runUnload;
+    using LibraryManager::prepareUnloadLibrary;
 };
 
 
+// Check that LibraryManager constructor requires a not null manager
+
+TEST_F(LibraryManagerTest, NullManager) {
+    boost::shared_ptr<CalloutManager> null_manager;
+    EXPECT_THROW(PublicLibraryManager(std::string("foo"), 0, null_manager),
+                 NoCalloutManager);
+}
+
 // Check that openLibrary() reports an error when it can't find the specified
 // library.
 
@@ -215,6 +247,90 @@ TEST_F(LibraryManagerTest, CorrectVersionReturned) {
     EXPECT_TRUE(lib_manager.closeLibrary());
 }
 
+// Checks that the code handles the case of a library with no
+// multi_threading_compatible function.
+
+TEST_F(LibraryManagerTest, NoMultiThreadingCompatible) {
+    PublicLibraryManager lib_manager(std::string(BASIC_CALLOUT_LIBRARY),
+                                     0, callout_manager_);
+
+    // Open should succeed.
+    EXPECT_TRUE(lib_manager.openLibrary());
+
+    // Not multi-threading compatible: does not matter without MT.
+    EXPECT_TRUE(lib_manager.checkMultiThreadingCompatible());
+
+    // Not multi-threading compatible: does matter with MT.
+    MultiThreadingMgr::instance().setMode(true);
+    EXPECT_FALSE(lib_manager.checkMultiThreadingCompatible());
+
+    // Tidy up.
+    EXPECT_TRUE(lib_manager.closeLibrary());
+}
+
+// Checks that the code handles the case of a library with a
+// multi_threading_compatible function returning 0 (not compatible).
+
+TEST_F(LibraryManagerTest, multiThreadingNotCompatible) {
+    PublicLibraryManager lib_manager(std::string(LOAD_ERROR_CALLOUT_LIBRARY),
+                                     0, callout_manager_);
+
+    // Open should succeed.
+    EXPECT_TRUE(lib_manager.openLibrary());
+
+    // Not multi-threading compatible: does not matter without MT.
+    EXPECT_TRUE(lib_manager.checkMultiThreadingCompatible());
+
+    // Not multi-threading compatible: does matter with MT.
+    MultiThreadingMgr::instance().setMode(true);
+    EXPECT_FALSE(lib_manager.checkMultiThreadingCompatible());
+
+    // Tidy up.
+    EXPECT_TRUE(lib_manager.closeLibrary());
+}
+
+// Checks that the code handles the case of a library with a
+// multi_threading_compatible function returning 1 (compatible)
+
+TEST_F(LibraryManagerTest, multiThreadingCompatible) {
+    PublicLibraryManager lib_manager(std::string(FULL_CALLOUT_LIBRARY),
+                                     0, callout_manager_);
+
+    // Open should succeed.
+    EXPECT_TRUE(lib_manager.openLibrary());
+
+    // Multi-threading compatible: does not matter without MT.
+    EXPECT_TRUE(lib_manager.checkMultiThreadingCompatible());
+
+    // Multi-threading compatible: does matter with MT.
+    MultiThreadingMgr::instance().setMode(true);
+    EXPECT_TRUE(lib_manager.checkMultiThreadingCompatible());
+
+    // Tidy up.
+    EXPECT_TRUE(lib_manager.closeLibrary());
+}
+
+// Checks that the code handles the case of a library with a
+// multi_threading_compatible function returning 1 (compatible)
+
+TEST_F(LibraryManagerTest, multiThreadingCompatibleException) {
+    PublicLibraryManager lib_manager(std::string(FRAMEWORK_EXCEPTION_LIBRARY),
+                                     0, callout_manager_);
+
+    // Open should succeed.
+    EXPECT_TRUE(lib_manager.openLibrary());
+
+    // Throw exception: does not matter without MT.
+    EXPECT_TRUE(lib_manager.checkMultiThreadingCompatible());
+
+    // Throw exception: does matter with MT.
+    MultiThreadingMgr::instance().setMode(true);
+    EXPECT_FALSE(lib_manager.checkMultiThreadingCompatible());
+
+    // Tidy up.
+    EXPECT_TRUE(lib_manager.closeLibrary());
+}
+
 // Checks the registration of standard callouts.
 
 TEST_F(LibraryManagerTest, RegisterStandardCallouts) {
@@ -222,7 +338,7 @@ TEST_F(LibraryManagerTest, RegisterStandardCallouts) {
     // Load the only library, specifying the index of 0 as it's the only
     // library.  This should load all callouts.
     PublicLibraryManager lib_manager(std::string(BASIC_CALLOUT_LIBRARY),
-                               0, callout_manager_);
+                                     0, callout_manager_);
     EXPECT_TRUE(lib_manager.openLibrary());
 
     // Check the version of the library.
@@ -263,6 +379,8 @@ TEST_F(LibraryManagerTest, CheckLoadCalled) {
     EXPECT_TRUE(callout_manager_->calloutsPresent(hookpt_one_index_));
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_two_index_));
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_three_index_));
+    EXPECT_FALSE(callout_manager_->commandHandlersPresent("command-one"));
+    EXPECT_FALSE(callout_manager_->commandHandlersPresent("command-two"));
     EXPECT_FALSE(callout_manager_->calloutsPresent(
                  ServerHooks::CONTEXT_DESTROY));
 
@@ -273,6 +391,8 @@ TEST_F(LibraryManagerTest, CheckLoadCalled) {
     EXPECT_TRUE(callout_manager_->calloutsPresent(hookpt_one_index_));
     EXPECT_TRUE(callout_manager_->calloutsPresent(hookpt_two_index_));
     EXPECT_TRUE(callout_manager_->calloutsPresent(hookpt_three_index_));
+    EXPECT_TRUE(callout_manager_->commandHandlersPresent("command-one"));
+    EXPECT_TRUE(callout_manager_->commandHandlersPresent("command-two"));
     EXPECT_FALSE(callout_manager_->calloutsPresent(
                  ServerHooks::CONTEXT_DESTROY));
 
@@ -282,6 +402,11 @@ TEST_F(LibraryManagerTest, CheckLoadCalled) {
     // r3 = (5 * d1 + d2) * d3
     executeCallCallouts(5, 5, 25, 7, 32, 10, 320);
 
+    // Execute command handlers for 'command-one' and 'command-two'.
+    //
+    // r2 = d1 * d2 * 10;
+    executeCallCommandHandlers(5, 5, 7, 350);
+
     // Tidy up
     EXPECT_TRUE(lib_manager.closeLibrary());
 }
@@ -331,7 +456,7 @@ TEST_F(LibraryManagerTest, CheckNoUnload) {
     EXPECT_TRUE(lib_manager.openLibrary());
 
     // Check that no unload function returns true.
-    EXPECT_TRUE(lib_manager.runUnload());
+    EXPECT_TRUE(lib_manager.prepareUnloadLibrary());
 
     // Tidy up
     EXPECT_TRUE(lib_manager.closeLibrary());
@@ -348,7 +473,7 @@ TEST_F(LibraryManagerTest, CheckUnloadError) {
     EXPECT_TRUE(lib_manager.openLibrary());
 
     // Check that unload function returning an error returns false.
-    EXPECT_FALSE(lib_manager.runUnload());
+    EXPECT_FALSE(lib_manager.prepareUnloadLibrary());
 
     // Tidy up
     EXPECT_TRUE(lib_manager.closeLibrary());
@@ -365,7 +490,7 @@ TEST_F(LibraryManagerTest, CheckUnloadException) {
     EXPECT_TRUE(lib_manager.openLibrary());
 
     // Check that we detect that the unload function throws an exception.
-    EXPECT_FALSE(lib_manager.runUnload());
+    EXPECT_FALSE(lib_manager.prepareUnloadLibrary());
 
     // Tidy up
     EXPECT_TRUE(lib_manager.closeLibrary());
@@ -388,7 +513,7 @@ TEST_F(LibraryManagerTest, CheckUnload) {
     EXPECT_FALSE(markerFilePresent());
 
     // Check that unload function runs and returns a success
-    EXPECT_TRUE(lib_manager.runUnload());
+    EXPECT_TRUE(lib_manager.prepareUnloadLibrary());
 
     // Check that the marker file was created.
     EXPECT_TRUE(markerFilePresent());
@@ -406,8 +531,8 @@ TEST_F(LibraryManagerTest, LibUnload) {
 
     // Load the only library, specifying the index of 0 as it's the only
     // library.  This should load all callouts.
-    PublicLibraryManager lib_manager(std::string(FULL_CALLOUT_LIBRARY),
-                               0, callout_manager_);
+    PublicLibraryManager lib_manager(std::string(LOAD_CALLOUT_LIBRARY),
+                                     0, callout_manager_);
     EXPECT_TRUE(lib_manager.openLibrary());
 
     // Check the version of the library.
@@ -417,18 +542,24 @@ TEST_F(LibraryManagerTest, LibUnload) {
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_one_index_));
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_two_index_));
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_three_index_));
+    EXPECT_FALSE(callout_manager_->commandHandlersPresent("command-one"));
+    EXPECT_FALSE(callout_manager_->commandHandlersPresent("command-two"));
 
     // Load the single standard callout and check it is registered correctly.
     EXPECT_NO_THROW(lib_manager.registerStandardCallouts());
     EXPECT_TRUE(callout_manager_->calloutsPresent(hookpt_one_index_));
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_two_index_));
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_three_index_));
+    EXPECT_FALSE(callout_manager_->commandHandlersPresent("command-one"));
+    EXPECT_FALSE(callout_manager_->commandHandlersPresent("command-two"));
 
     // Call the load function to load the other callouts.
     EXPECT_TRUE(lib_manager.runLoad());
     EXPECT_TRUE(callout_manager_->calloutsPresent(hookpt_one_index_));
     EXPECT_TRUE(callout_manager_->calloutsPresent(hookpt_two_index_));
     EXPECT_TRUE(callout_manager_->calloutsPresent(hookpt_three_index_));
+    EXPECT_TRUE(callout_manager_->commandHandlersPresent("command-one"));
+    EXPECT_TRUE(callout_manager_->commandHandlersPresent("command-two"));
 
     // Unload the library and check that the callouts have been removed from
     // the CalloutManager.
@@ -436,6 +567,8 @@ TEST_F(LibraryManagerTest, LibUnload) {
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_one_index_));
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_two_index_));
     EXPECT_FALSE(callout_manager_->calloutsPresent(hookpt_three_index_));
+    EXPECT_FALSE(callout_manager_->commandHandlersPresent("command-one"));
+    EXPECT_FALSE(callout_manager_->commandHandlersPresent("command-two"));
 }
 
 // Now come the loadLibrary() tests that make use of all the methods tested
@@ -471,8 +604,8 @@ TEST_F(LibraryManagerTest, LoadLibraryWrongVersion) {
 // Check that the full loadLibrary call works.
 
 TEST_F(LibraryManagerTest, LoadLibrary) {
-    LibraryManager lib_manager(std::string(FULL_CALLOUT_LIBRARY), 0,
-                                           callout_manager_);
+    PublicLibraryManager lib_manager(std::string(FULL_CALLOUT_LIBRARY), 0,
+                                     callout_manager_);
     EXPECT_TRUE(lib_manager.loadLibrary());
 
     // Now execute the callouts in the order expected.  The library performs
@@ -499,28 +632,28 @@ TEST_F(LibraryManagerTest, LoadLibrary) {
 
 TEST_F(LibraryManagerTest, LoadMultipleLibraries) {
     // Load a library with all framework functions.
-    LibraryManager lib_manager_1(std::string(FULL_CALLOUT_LIBRARY), 0,
-                                 callout_manager_);
+    PublicLibraryManager lib_manager_1(std::string(FULL_CALLOUT_LIBRARY),
+                                       0, callout_manager_);
     EXPECT_TRUE(lib_manager_1.loadLibrary());
 
     // Attempt to load a library with no version() function.  We should detect
     // this and not end up calling the function from the already loaded
     // library.
-    LibraryManager lib_manager_2(std::string(NO_VERSION_LIBRARY), 1,
-                                 callout_manager_);
+    PublicLibraryManager lib_manager_2(std::string(NO_VERSION_LIBRARY),
+                                       1, callout_manager_);
     EXPECT_FALSE(lib_manager_2.loadLibrary());
 
     // Attempt to load the library with an incorrect version.  This should
     // be detected.
-    LibraryManager lib_manager_3(std::string(INCORRECT_VERSION_LIBRARY), 1,
-                                 callout_manager_);
+    PublicLibraryManager lib_manager_3(std::string(INCORRECT_VERSION_LIBRARY),
+                                       1, callout_manager_);
     EXPECT_FALSE(lib_manager_3.loadLibrary());
 
     // Load the basic callout library.  This only has standard callouts so,
     // if the first library's load() function gets called, some callouts
     // will be registered twice and lead to incorrect results.
-    LibraryManager lib_manager_4(std::string(BASIC_CALLOUT_LIBRARY), 1,
-                                 callout_manager_);
+    PublicLibraryManager lib_manager_4(std::string(BASIC_CALLOUT_LIBRARY),
+                                       1, callout_manager_);
     EXPECT_TRUE(lib_manager_4.loadLibrary());
 
     // Execute the callouts.  The first library implements the calculation.
@@ -561,18 +694,31 @@ TEST_F(LibraryManagerTest, validateLibraries) {
     EXPECT_FALSE(LibraryManager::validateLibrary(NO_VERSION_LIBRARY));
     EXPECT_TRUE(LibraryManager::validateLibrary(UNLOAD_CALLOUT_LIBRARY));
     EXPECT_TRUE(LibraryManager::validateLibrary(CALLOUT_PARAMS_LIBRARY));
+
+    MultiThreadingMgr::instance().setMode(true);
+
+    EXPECT_FALSE(LibraryManager::validateLibrary(BASIC_CALLOUT_LIBRARY));
+    EXPECT_TRUE(LibraryManager::validateLibrary(FULL_CALLOUT_LIBRARY));
+    EXPECT_FALSE(LibraryManager::validateLibrary(FRAMEWORK_EXCEPTION_LIBRARY));
+    EXPECT_FALSE(LibraryManager::validateLibrary(INCORRECT_VERSION_LIBRARY));
+    EXPECT_FALSE(LibraryManager::validateLibrary(LOAD_CALLOUT_LIBRARY));
+    EXPECT_FALSE(LibraryManager::validateLibrary(LOAD_ERROR_CALLOUT_LIBRARY));
+    EXPECT_FALSE(LibraryManager::validateLibrary(NOT_PRESENT_LIBRARY));
+    EXPECT_FALSE(LibraryManager::validateLibrary(NO_VERSION_LIBRARY));
+    EXPECT_FALSE(LibraryManager::validateLibrary(UNLOAD_CALLOUT_LIBRARY));
+    EXPECT_FALSE(LibraryManager::validateLibrary(CALLOUT_PARAMS_LIBRARY));
 }
 
 // Check that log messages are properly registered and unregistered.
 
 TEST_F(LibraryManagerTest, libraryLoggerSetup) {
     // Load a library with all framework functions.
-    LibraryManager lib_manager(std::string(BASIC_CALLOUT_LIBRARY), 0,
-                               callout_manager_);
+    PublicLibraryManager lib_manager(std::string(BASIC_CALLOUT_LIBRARY),
+                                     0, callout_manager_);
     EXPECT_TRUE(lib_manager.loadLibrary());
 
     // After loading the library, the global logging dictionary should
-    // contain log messages registerd for this library.
+    // contain log messages registered for this library.
     const MessageDictionaryPtr& dict = MessageDictionary::globalDictionary();
     EXPECT_EQ("basic callout load %1", dict->getText("BCL_LOAD_START"));
     EXPECT_EQ("basic callout load end", dict->getText("BCL_LOAD_END"));
@@ -585,8 +731,11 @@ TEST_F(LibraryManagerTest, libraryLoggerSetup) {
 
     // After unloading the library, the messages should be unregistered.
     EXPECT_TRUE(lib_manager.unloadLibrary());
+    // The musl libc does not implement dlclose
+#ifndef LIBC_MUSL
     EXPECT_TRUE(dict->getText("BCL_LOAD_START").empty());
     EXPECT_TRUE(dict->getText("BCL_LOAD_END").empty());
+#endif
 }
 
 } // Anonymous namespace
