@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,9 +6,9 @@
 
 #include <config.h>
 
+#include <database/db_exceptions.h>
 #include <database/db_log.h>
 #include <pgsql/pgsql_connection.h>
-#include <pgsql/pgsql_exchange.h>
 
 // PostgreSQL errors should be tested based on the SQL state code.  Each state
 // code is 5 decimal, ASCII, digits, the first two define the category of
@@ -39,6 +39,9 @@ namespace db {
 const int PGSQL_DEFAULT_CONNECTION_TIMEOUT = 5; // seconds
 
 const char PgSqlConnection::DUPLICATE_KEY[] = ERRCODE_UNIQUE_VIOLATION;
+const char PgSqlConnection::NULL_KEY[] = ERRCODE_NOT_NULL_VIOLATION;
+
+bool PgSqlConnection::warned_about_tls = false;
 
 PgSqlResult::PgSqlResult(PGresult *result)
     : result_(result), rows_(0), cols_(0) {
@@ -161,7 +164,9 @@ PgSqlConnection::prepareStatement(const PgSqlTaggedStatement& statement) {
                             statement.nbparams, statement.types));
     if (PQresultStatus(r) != PGRES_COMMAND_OK) {
         isc_throw(DbOperationError, "unable to prepare PostgreSQL statement: "
-                  << statement.text << ", reason: " << PQerrorMessage(conn_));
+                  << " name: " << statement.name
+                  << ", reason: " << PQerrorMessage(conn_)
+                  << ", text: " << statement.text);
     }
 }
 
@@ -175,8 +180,8 @@ PgSqlConnection::prepareStatements(const PgSqlTaggedStatement* start_statement,
     }
 }
 
-void
-PgSqlConnection::openDatabase() {
+std::string
+PgSqlConnection::getConnParameters() {
     string dbconnparameters;
     string shost = "localhost";
     try {
@@ -187,38 +192,19 @@ PgSqlConnection::openDatabase() {
 
     dbconnparameters += "host = '" + shost + "'" ;
 
-    string sport;
+    unsigned int port = 0;
     try {
-        sport = getParameter("port");
-    } catch (...) {
-        // No port parameter, we are going to use the default port.
-        sport = "";
+        setIntParameterValue("port", 0, numeric_limits<uint16_t>::max(), port);
+
+    } catch (const std::exception& ex) {
+        isc_throw(DbInvalidPort, ex.what());
     }
 
-    if (sport.size() > 0) {
-        unsigned int port = 0;
-
-        // Port was given, so try to convert it to an integer.
-        try {
-            port = boost::lexical_cast<unsigned int>(sport);
-        } catch (...) {
-            // Port given but could not be converted to an unsigned int.
-            // Just fall back to the default value.
-            port = 0;
-        }
-
-        // The port is only valid when it is in the 0..65535 range.
-        // Again fall back to the default when the given value is invalid.
-        if (port > numeric_limits<uint16_t>::max()) {
-            port = 0;
-        }
-
-        // Add it to connection parameters when not default.
-        if (port > 0) {
-            std::ostringstream oss;
-            oss << port;
-            dbconnparameters += " port = " + oss.str();
-        }
+    // Add port to connection parameters when not default.
+    if (port > 0) {
+        std::ostringstream oss;
+        oss << port;
+        dbconnparameters += " port = " + oss.str();
     }
 
     string suser;
@@ -247,46 +233,36 @@ PgSqlConnection::openDatabase() {
     }
 
     unsigned int connect_timeout = PGSQL_DEFAULT_CONNECTION_TIMEOUT;
-    string stimeout;
+    unsigned int tcp_user_timeout = 0;
     try {
-        stimeout = getParameter("connect-timeout");
-    } catch (...) {
-        // No timeout parameter, we are going to use the default timeout.
-        stimeout = "";
-    }
-
-    if (stimeout.size() > 0) {
-        // Timeout was given, so try to convert it to an integer.
-
-        try {
-            connect_timeout = boost::lexical_cast<unsigned int>(stimeout);
-        } catch (...) {
-            // Timeout given but could not be converted to an unsigned int. Set
-            // the connection timeout to an invalid value to trigger throwing
-            // of an exception.
-            connect_timeout = 0;
-        }
-
         // The timeout is only valid if greater than zero, as depending on the
         // database, a zero timeout might signify something like "wait
         // indefinitely".
-        //
-        // The check below also rejects a value greater than the maximum
-        // integer value.  The lexical_cast operation used to obtain a numeric
-        // value from a string can get confused if trying to convert a negative
-        // integer to an unsigned int: instead of throwing an exception, it may
-        // produce a large positive value.
-        if ((connect_timeout == 0) ||
-            (connect_timeout > numeric_limits<int>::max())) {
-            isc_throw(DbInvalidTimeout, "database connection timeout (" <<
-                      stimeout << ") must be an integer greater than 0");
-        }
+        setIntParameterValue("connect-timeout", 1, numeric_limits<int>::max(), connect_timeout);
+        // This timeout value can be 0, meaning that the database client will
+        // follow a default behavior. Earlier Postgres versions didn't have
+        // this parameter, so we allow 0 to skip setting them for these
+        // earlier versions.
+        setIntParameterValue("tcp-user-timeout", 0, numeric_limits<int>::max(), tcp_user_timeout);
+
+    } catch (const std::exception& ex) {
+        isc_throw(DbInvalidTimeout, ex.what());
     }
 
+    // Append timeouts.
     std::ostringstream oss;
-    oss << connect_timeout;
-    dbconnparameters += " connect_timeout = " + oss.str();
+    oss << " connect_timeout = " << connect_timeout;
+    if (tcp_user_timeout > 0) {
+        oss << " tcp_user_timeout = " << tcp_user_timeout * 1000;
+    }
+    dbconnparameters += oss.str();
 
+    return (dbconnparameters);
+}
+
+void
+PgSqlConnection::openDatabase() {
+    std::string dbconnparameters = getConnParameters();
     // Connect to Postgres, saving the low level connection pointer
     // in the holder object
     PGconn* new_conn = PQconnectdb(dbconnparameters.c_str());
@@ -347,6 +323,18 @@ PgSqlConnection::checkStatementError(const PgSqlResult& r,
                       "fatal database error or connectivity lost");
         }
 
+        // Failure: check for the special case of duplicate entry.
+        if (compareError(r, PgSqlConnection::DUPLICATE_KEY)) {
+            isc_throw(DuplicateEntry, "statement: " << statement.name
+                      << ", reason: " << PQerrorMessage(conn_));
+        }
+
+        // Failure: check for the special case of null key violation.
+        if (compareError(r, PgSqlConnection::NULL_KEY)) {
+            isc_throw(NullKeyError, "statement: " << statement.name
+                      << ", reason: " << PQerrorMessage(conn_));
+        }
+
         // Apparently it wasn't fatal, so we throw with a helpful message.
         const char* error_message = PQerrorMessage(conn_);
         isc_throw(DbOperationError, "Statement exec failed for: "
@@ -358,6 +346,11 @@ PgSqlConnection::checkStatementError(const PgSqlResult& r,
 
 void
 PgSqlConnection::startTransaction() {
+    // If it is nested transaction, do nothing.
+    if (++transaction_ref_count_ > 1) {
+        return;
+    }
+
     DB_LOG_DEBUG(DB_DBG_TRACE_DETAIL, PGSQL_START_TRANSACTION);
     checkUnusable();
     PgSqlResult r(PQexec(conn_, "START TRANSACTION"));
@@ -368,8 +361,22 @@ PgSqlConnection::startTransaction() {
     }
 }
 
+bool
+PgSqlConnection::isTransactionStarted() const {
+    return (transaction_ref_count_ > 0);
+}
+
 void
 PgSqlConnection::commit() {
+    if (transaction_ref_count_ <= 0) {
+        isc_throw(Unexpected, "commit called for not started transaction - coding error");
+    }
+
+    // When committing nested transaction, do nothing.
+    if (--transaction_ref_count_ > 0) {
+        return;
+    }
+
     DB_LOG_DEBUG(DB_DBG_TRACE_DETAIL, PGSQL_COMMIT);
     checkUnusable();
     PgSqlResult r(PQexec(conn_, "COMMIT"));
@@ -381,6 +388,15 @@ PgSqlConnection::commit() {
 
 void
 PgSqlConnection::rollback() {
+    if (transaction_ref_count_ <= 0) {
+        isc_throw(Unexpected, "rollback called for not started transaction - coding error");
+    }
+
+    // When rolling back nested transaction, do nothing.
+    if (--transaction_ref_count_ > 0) {
+        return;
+    }
+
     DB_LOG_DEBUG(DB_DBG_TRACE_DETAIL, PGSQL_ROLLBACK);
     checkUnusable();
     PgSqlResult r(PQexec(conn_, "ROLLBACK"));
@@ -390,5 +406,136 @@ PgSqlConnection::rollback() {
     }
 }
 
+void
+PgSqlConnection::createSavepoint(const std::string& name) {
+    if (transaction_ref_count_ <= 0) {
+        isc_throw(InvalidOperation, "no transaction, cannot create savepoint: " << name);
+    }
+
+    DB_LOG_DEBUG(DB_DBG_TRACE_DETAIL, PGSQL_CREATE_SAVEPOINT).arg(name);
+    std::string sql("SAVEPOINT " + name);
+    executeSQL(sql);
+}
+
+void
+PgSqlConnection::rollbackToSavepoint(const std::string& name) {
+    if (transaction_ref_count_ <= 0) {
+        isc_throw(InvalidOperation, "no transaction, cannot rollback to savepoint: " << name);
+    }
+
+    std::string sql("ROLLBACK TO SAVEPOINT " + name);
+    executeSQL(sql);
+}
+
+void
+PgSqlConnection::executeSQL(const std::string& sql) {
+    // Use a TaggedStatement so we can call checkStatementError and ensure
+    // we detect connectivity issues properly.
+    PgSqlTaggedStatement statement({0, {OID_NONE}, "run-statement", sql.c_str()});
+    checkUnusable();
+    PgSqlResult r(PQexec(conn_, statement.text));
+    checkStatementError(r, statement);
+}
+
+PgSqlResultPtr
+PgSqlConnection::executePreparedStatement(PgSqlTaggedStatement& statement,
+                                          const PsqlBindArray& in_bindings) {
+    checkUnusable();
+
+    if (statement.nbparams != in_bindings.size()) {
+        isc_throw (InvalidOperation, "executePreparedStatement:"
+                   << " expected: " << statement.nbparams
+                   << " parameters, given: " << in_bindings.size()
+                   << ", statement: " << statement.name
+                   << ", SQL: " << statement.text);
+    }
+
+    const char* const* values = 0;
+    const int* lengths = 0;
+    const int* formats = 0;
+    if (statement.nbparams > 0) {
+        values = static_cast<const char* const*>(&in_bindings.values_[0]);
+        lengths = static_cast<const int *>(&in_bindings.lengths_[0]);
+        formats = static_cast<const int *>(&in_bindings.formats_[0]);
+    }
+
+    PgSqlResultPtr result_set;
+    result_set.reset(new PgSqlResult(PQexecPrepared(conn_, statement.name, statement.nbparams,
+                                                    values, lengths, formats, 0)));
+
+    checkStatementError(*result_set, statement);
+    return (result_set);
+}
+
+void
+PgSqlConnection::selectQuery(PgSqlTaggedStatement& statement,
+                             const PsqlBindArray& in_bindings,
+                             ConsumeResultRowFun process_result_row) {
+    // Execute the prepared statement.
+    PgSqlResultPtr result_set = executePreparedStatement(statement, in_bindings);
+
+    // Iterate over the returned rows and invoke the row consumption
+    // function on each one.
+    int rows = result_set->getRows();
+    for (int row = 0; row < rows; ++row) {
+        try {
+            process_result_row(*result_set, row);
+        } catch (const std::exception& ex) {
+            // Rethrow the exception with a bit more data.
+            isc_throw(BadValue, ex.what() << ". Statement is <" <<
+                      statement.text << ">");
+        }
+    }
+}
+
+void
+PgSqlConnection::insertQuery(PgSqlTaggedStatement& statement,
+                             const PsqlBindArray& in_bindings) {
+    // Execute the prepared statement.
+    PgSqlResultPtr result_set = executePreparedStatement(statement, in_bindings);
+}
+
+uint64_t
+PgSqlConnection::updateDeleteQuery(PgSqlTaggedStatement& statement,
+                                   const PsqlBindArray& in_bindings) {
+    // Execute the prepared statement.
+    PgSqlResultPtr result_set = executePreparedStatement(statement, in_bindings);
+
+    return (boost::lexical_cast<int>(PQcmdTuples(*result_set)));
+}
+
+template<typename T>
+void
+PgSqlConnection::setIntParameterValue(const std::string& name, int64_t min, int64_t max, T& value) {
+    string svalue;
+    try {
+        svalue = getParameter(name);
+    } catch (...) {
+        // Do nothing if the parameter is not present.
+    }
+    if (svalue.empty()) {
+        return;
+    }
+    try {
+        // Try to convert the value.
+        auto parsed_value = boost::lexical_cast<T>(svalue);
+        // Check if the value is within the specified range.
+        if ((parsed_value < min) || (parsed_value > max)) {
+            isc_throw(BadValue, "bad " << svalue << " value");
+        }
+        // Everything is fine. Return the parsed value.
+        value = parsed_value;
+
+    } catch (...) {
+        // We may end up here when lexical_cast fails or when the
+        // parsed value is not within the desired range. In both
+        // cases let's throw the same general error.
+        isc_throw(BadValue, name << " parameter (" <<
+                  svalue << ") must be an integer between "
+                  << min << " and " << max);
+    }
+}
+
+
 } // end of isc::db namespace
 } // end of isc namespace
